/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2014, 2015 (c)
 * 
 *    file: grid.c
 *    This file is part of the GoSnake project.
 *
 *    GoSnake is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    GoSnake is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with GoSnake.  If not, see <http://www.gnu.org/licenses/>.
 */    

#include "defs.h"

extern char *grid_buf;

void create_outer_border(int j)
{
  int i;
  memset(grid_buf, 0, (SCREEN_H+1)*(SCREEN_W+1));
  for(i = 1; i <= SCREEN_W; i++) grid_buf[SCREEN_W+i]   = 1;
  for(i = 1; i <= SCREEN_W; i++) grid_buf[i+j]          = 1;
  for(i = 1; i <= SCREEN_H; i++) grid_buf[i*SCREEN_W-1] = 1;
  for(i = 1; i <= SCREEN_H; i++) grid_buf[i*SCREEN_W]   = 1;
}

void make_grid(int level)
{
  int i, j = SCREEN_H*SCREEN_W, k, l;
  switch(level)
  {
    case 1:
      /*
       * Level #1 grid:
       * +-----------------+
       * |                 |
       * |                 |
       * |                 |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      break;
    case 2:
      /*
       * Level #2 grid:
       * +-----------------+
       * |                 |
       * |    |       |    |
       * |    |       |    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/3;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      l *= 2;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      break;
    case 3:
      /*
       * Level #3 grid:
       * +-----------------+
       * |                 |
       * |    ---------    |
       * |    ---------    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      k *= 2;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 4:
      /*
       * Level #4 grid:
       * +-----------------+
       * |                 |
       * |    +--------    |
       * |    +--------    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      k *= 2;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 5:
      /*
       * Level #5 grid:
       * +-----------------+
       * |                 |
       * |    ---   ---    |
       * |    ---   ---    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/5;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      k *= 2;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 6:
      /*
       * Level #6 grid:
       * +-----------------+
       * |                 |
       * |    |   |   |    |
       * |    |   |   |    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/4;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l]     = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+(l*2)] = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+(l*3)] = 1;
      break;
    case 7:
      /*
       * Level #7 grid:
       * +-----------------+
       * |                 |
       * |    +--------    |
       * |    +--   ---    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/5;
      for(i = l; i <= (l*4); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      k *= 2;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 8:
      /*
       * Level #8 grid:
       * +-----------------+
       * |                 |
       * |    ---   ---    |
       * |       ---       |
       * |    ---   ---    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/4;
      l = (SCREEN_W)/5;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i]   = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i]   = 1;
      for(i = (l*2); i <= (l*3); i++)grid_buf[k*2*SCREEN_W+i] = 1;
      k *= 3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i]   = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i]   = 1;
      break;
    case 9:
      /*
       * Level #9 grid:
       * +-----------------+
       * |                 |
       * |      ----       |
       * +----        -----+
       * |      ----       |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/4;
      l = (SCREEN_W)/3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      k *= 2;
      for(i = 1; i <= l; i++)        grid_buf[k*SCREEN_W+i] = 1;      
      for(i = (l*2); i <= (l*3); i++)grid_buf[k*SCREEN_W+i] = 1;
      k = (SCREEN_H)/4;
      k *= 3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 10:
      /*
       * Level #10 grid:
       * +-----------------+
       * |    |            |
       * |    |   |   |    |
       * |    |   |   |    |
       * |            |    |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/4;
      for(i = 1; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l]     = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+(l*2)] = 1;
      for(i = k; i <= (k*3); i++)    grid_buf[i*SCREEN_W+(l*3)] = 1;
      break;
    case 11:
      /*
       * Level #11 grid:
       * +-----------------+
       * |                 |
       * | |  |   |   |  | |
       * | |  |   |   |  | |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/6;
      for(i = k; i <= (k*2); i++)
      {
	grid_buf[i*SCREEN_W+l]     = 1;
	grid_buf[i*SCREEN_W+(l*2)] = 1;
	grid_buf[i*SCREEN_W+(l*3)] = 1;
	grid_buf[i*SCREEN_W+(l*4)] = 1;
	grid_buf[i*SCREEN_W+(l*5)] = 1;
      }
      break;
    case 12:
      /*
       * Level #12 grid:
       * +-----------------+
       * |                 |
       * |    +-------+    |
       * |    |       |    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      l *= 2;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      break;
    case 13:
      /*
       * Level #13 grid:
       * +-----------------+
       * |    ---------    |
       * |    ---------    |
       * |    ---------    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/5;
      l = (SCREEN_W)/3;
      int k2 = k;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      k *= 2;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      k = k2*3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      k = k2*4;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 14:
      /*
       * Level #14 grid:
       * +-----------------+
       * |                 |
       * |    |       |    |
       * |    +-------+    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/3;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      for(i = l; i <= (l*2); i++)    grid_buf[k*2*SCREEN_W+i] = 1;
      l *= 2;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      break;
    case 15:
      /*
       * Level #15 grid:
       * +-----------------+
       * |                 |
       * |    +-------+    |
       * |    +--   --+    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/5;
      for(i = l; i <= (l*4); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = k; i <= (k*2); i++)
      {
	grid_buf[i*SCREEN_W+l]     = 1;
	grid_buf[i*SCREEN_W+(l*4)] = 1;
      }
      k *= 2;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 16:
      /*
       * Level #16 grid:
       * +-----------------+
       * |                 |
       * |    +---+   |    |
       * |    |   +---+    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/4;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i]     = 1;
      for(i = (l*2); i <= (l*3); i++)grid_buf[k*2*SCREEN_W+i]   = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l]     = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+(l*2)] = 1;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+(l*3)] = 1;
      break;
    case 17:
      /*
       * Level #17 grid:
       * +-----------------+
       * |    ---   ---    |
       * |    ---   ---    |
       * |    ---   ---    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/5;
      l = (SCREEN_W)/5;
      int k1 = k;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      k *= 2;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      k = k1*3;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      k = k1*4;
      for(i = l; i <= (l*2); i++)    grid_buf[k*SCREEN_W+i] = 1;
      for(i = (l*3); i <= (l*4); i++)grid_buf[k*SCREEN_W+i] = 1;
      break;
    case 18:
      /*
       * Level #18 grid:
       * +-----------------+
       * |                 |
       * |    |       +----+
       * |    +-------+    |
       * |                 |
       * +-----------------+
       */
      create_outer_border(j);
      k = (SCREEN_H)/3;
      l = (SCREEN_W)/3;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      for(i = l; i <= (l*2); i++)    grid_buf[k*2*SCREEN_W+i] = 1;
      l *= 2;
      for(i = k; i <= (k*2); i++)    grid_buf[i*SCREEN_W+l] = 1;
      for(i = l; i <= SCREEN_W; i++) grid_buf[k*SCREEN_W+i] = 1;
      break;
  }//end switch(level)
}//end make_grid()


/* This is used for debugging */

/*
void dump_grid()
{
  FILE *o = fopen("output", "w");
  j = 0;
  int pos = SCREEN_W;
  fputc('\n', stdout);
  for(i = 0; i < SCREEN_H; i++)
  {
    printf("%d/%d  [", i+1, SCREEN_H);
    //fprintf(o, "%d/%d  [", i+1, SCREEN_H);
    for(j = 0; j < SCREEN_W; j++)
    {
      if(grid_buf[pos] == 0) { fputc(' ', o); fputc('_', stdout); }
      else { fputc('*', o); fputc('*', stdout); }
      pos++;
    //fwrite(grid_buf+j, SCREEN_W, 1, o);
    //fwrite("\n", 1, 1, o);
    //j++;
    //j += SCREEN_W;
    }
    fputc('\n', o);
    fputc('\n', stdout);
  }
  //while(getKey() != ENTER_KEY) ;
  fclose(o);
}
*/