/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2014, 2015 (c)
 * 
 *    file: main.c
 *    This file is part of the GoSnake project.
 *
 *    GoSnake is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    GoSnake is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with GoSnake.  If not, see <http://www.gnu.org/licenses/>.
 */    

#include <getopt.h>
#include "defs.h"

void sighandler(int signo) 
{
  
}

void draw_snake();
void game_over();
void make_food();
void exit_game();
void load_level(int level);
extern void make_grid(int level);
extern int show_menu(char *title);

char *grid_buf;
int speed = 50000;
int MAX_SCORE = 10;

char *gosnake_ver = "2.0";


int main(int argc, char **argv)
{
  ///////////////////////////////////////
  //parse command line arguments
  ///////////////////////////////////////
  int START_LEVEL = 0;
  int c;
  static struct option long_options[] =
  {
    {"version", no_argument,       0,         'v'},
    {"help",    no_argument,       0,         'h'},
    {"level", required_argument,   0,         'l'},
    {0, 0, 0, 0}
  };
  
  while(1)
  {
    int option_index=0;
    c = getopt_long(argc, argv, "vhl:", long_options, &option_index);
    if(c==-1) break;	//end of options
    
    switch(c)
    {
      case 0:
	break;
      case 'v':	//show version & exit
	printf("%s\n", gosnake_ver);
	exit(0); break;
      case 'h':	//show help & exit
	printf("The Console Snake Game, version %s\n"
	       "Usage: %s [options]\n\n"
	       "Options:\n"
	       "  -h, --help           Print help [this page]\n"
	       "  -v, --version        Display program version and quit\n"
	       "  -l, --level n        Start playing from level 'n'\n"
	       , gosnake_ver, argv[0]);
	exit(0); break;
      case 'l':
	START_LEVEL = atoi(optarg);
	if(START_LEVEL < 1 || START_LEVEL > MAX_LEVEL)
	{
	  printf("Level must be between 1 and %d\n", MAX_LEVEL);
	  exit(1); break;
	}
      case '?':
	break;
      default:
	printf("Unknown option: -%c\n", c);
	abort();
    }
  }
  ///////////////////////////////////////
  ///////////////////////////////////////
  if(!initTerminal())
  {
    printf("Error initializing the screen\n");
    exit(1);
  }
  int flags;
  /* make stdin non-blocking */
  flags = fcntl(0, F_GETFL);
  flags |= O_NONBLOCK;
  fcntl(0, F_SETFL, flags);

  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  clearScreen();
  getScreenSize();
  drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
  //turn the cursor off
  printf("\e[?25l");
  
  //initialize
  grid_buf = (char *)malloc((SCREEN_H+1)*(SCREEN_W+1));
  if(!grid_buf)
  {
    printf("Error: Insufficient memory\n");
    exit_game();
  }
  int i, j;
  char grid[SCREEN_H+1][SCREEN_W+1];
  if(START_LEVEL)
  {
    load_level(START_LEVEL);
    memcpy(&grid, grid_buf, (SCREEN_H+1)*(SCREEN_W+1));
  }
  else
  {
    i = show_menu("GoSnake");
    if(i) memcpy(&grid, grid_buf, (SCREEN_H+1)*(SCREEN_W+1));
  }
  srand(time(NULL));
  make_food();
  locate(1, 3); 	  printf("SCORE: %04d", score);
  locate(1, SCREEN_W-10); printf("LEVEL: %02d", LEVEL);

  //Loop until we get a hit on ENTER
  long int wait = speed*3;
  while(1)
  {
    //eat food!
    if(snake.x == food.x && snake.y == food.y)
    { 
      score += food.score_inc;
      level_score += food.score_inc;
      if(snake.length < MAX_LENGTH) snake.length++;
      snake.body[snake.length-1] = snake.body[snake.length-2];
      make_food(); 
      ///////////////////////////////////////
      //check if we reached the target score
      ///////////////////////////////////////
      if(level_score >= MAX_SCORE)
      {
	//yes? goto next level
	LEVEL++;
	if(LEVEL > MAX_LEVEL) LEVEL = 1;
	MAX_SCORE += 5;
	speed -= 5000;
	level_score = 0;
	snake.length = 3;
	snake.x = SCREEN_H-5;
	snake.y = (SCREEN_W/2);
	for(i = 0; i < snake.length; i++) snake.body[i] = UP;
	snake.body_x[2] = snake.x;   snake.body_y[2] = snake.y;
	snake.body_x[1] = snake.x+1; snake.body_y[1] = snake.y;
	snake.body_x[0] = snake.x+2; snake.body_y[0] = snake.y;
	make_grid(LEVEL);	//make grid for new level
	memcpy(&grid, grid_buf, (SCREEN_H+1)*(SCREEN_W+1));
	drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
	draw_snake();
	usleep(speed);
      }
    }
    
    if(--wait) goto skip;
    wait = speed*3;

    if(snake.body[snake.length-1] == UP) snake.x--;
    else if(snake.body[snake.length-1] == DOWN) snake.x++;
    else if(snake.body[snake.length-1] == LEFT_1 ||
	    snake.body[snake.length-1] == LEFT_2) snake.y--;
    else if(snake.body[snake.length-1] == RIGHT_1 ||
	    snake.body[snake.length-1] == RIGHT_2) snake.y++;
    for(i = snake.length-2; i >= 0; i--)
      if(snake.body[i] != snake.body[i+1])
      {
	for(j = 0; j <= i; j++)
	  snake.body[j] = snake.body[j+1];
	break;
      }
      
    /**** check for wall hits ****/
    if(grid_buf[(SCREEN_W*snake.x)+snake.y-1])
    {
      game_over();
      continue;
    }
    
    draw_snake();
    
    int ch;
skip:
    ch = getKey();
    if(ch == LEFT_KEY || ch == 'a')
    {
      if(snake.body[snake.length-1] == UP)
      {
	/* correct the glitch that happens when user presser two keys quickly */
	if(snake.body[snake.length-2] == RIGHT_1 || snake.body[snake.length-2] == RIGHT_2)
	  snake.body[snake.length-2] = snake.body[snake.length-1];
	snake.body[snake.length-1] = LEFT_1;
      }
      else if(snake.body[snake.length-1] == DOWN)
      {
	/* correct the glitch that happens when user presser two keys quickly */
	if(snake.body[snake.length-2] == RIGHT_1 || snake.body[snake.length-2] == RIGHT_2)
	  snake.body[snake.length-2] = snake.body[snake.length-1];
	snake.body[snake.length-1] = LEFT_2;
      }
      else goto sleep;
      snake.y--; draw_snake();
    }
    else if(ch == RIGHT_KEY || ch == 'd')
    {
      if(snake.body[snake.length-1] == UP)
      {
	/* correct the glitch that happens when user presser two keys quickly */
	if(snake.body[snake.length-2] == LEFT_1 || snake.body[snake.length-2] == LEFT_2)
	  snake.body[snake.length-2] = snake.body[snake.length-1];
	snake.body[snake.length-1] = RIGHT_1;
      }
      else if(snake.body[snake.length-1] == DOWN)
      {
	/* correct the glitch that happens when user presser two keys quickly */
	if(snake.body[snake.length-2] == LEFT_1 || snake.body[snake.length-2] == LEFT_2)
	  snake.body[snake.length-2] = snake.body[snake.length-1];
	snake.body[snake.length-1] = RIGHT_2;
      }
      else goto sleep;
      snake.y++; draw_snake();
    }
    else if(ch == UP_KEY || ch == 'w')
    {
      if(snake.body[snake.length-1] == RIGHT_1 || snake.body[snake.length-1] == RIGHT_2
	 || snake.body[snake.length-1] == LEFT_1 || snake.body[snake.length-1] == LEFT_2)
      {
	/* correct the glitch that happens when user presser two keys quickly */
	if(snake.body[snake.length-2] == DOWN)
	  snake.body[snake.length-2] = snake.body[snake.length-1];
	snake.body[snake.length-1] = UP;
      }
      else goto sleep;
      snake.x--; draw_snake();
    }
    else if(ch == DOWN_KEY || ch == 's')
    {
      if(snake.body[snake.length-1] == RIGHT_1 || snake.body[snake.length-1] == RIGHT_2
	 || snake.body[snake.length-1] == LEFT_1 || snake.body[snake.length-1] == LEFT_2)
      {
	/* correct the glitch that happens when user presser two keys quickly */
	if(snake.body[snake.length-2] == UP)
	  snake.body[snake.length-2] = snake.body[snake.length-1];
	snake.body[snake.length-1] = DOWN;
      }
      else goto sleep;
      snake.x++; draw_snake();
    }
    else if(ch == ENTER_KEY || ch == SPACE_KEY || ch == 'p')
    {
      i = show_menu("PAUSED");
      if(i) memcpy(&grid, grid_buf, (SCREEN_H+1)*(SCREEN_W+1));
    }
    else if(ch == '0')
    {
      speed -= SPEED_STEP;
      if(speed < MAX_SPEED) speed = MAX_SPEED;
    }
    else if(ch == '9')
    {
      speed += SPEED_STEP;
      if(speed > MIN_SPEED) speed = MIN_SPEED;
    }
    else if(ch == ESC_KEY) break;
sleep:
    continue;
    //usleep(speed);
  }//end while

  exit_game();
  return 0;
}


/*
 * Draw the snake to the screen
 */
void draw_snake()
{
  //check for wall hits
  if(grid_buf[(SCREEN_W*snake.x)+snake.y-1])
  {
    game_over();
    return;
  }
  locate(1, 3); 	  printf("SCORE: %04d", score);
  locate(1, SCREEN_W-10); printf("LEVEL: %02d", LEVEL);
  setScreenColors(BLACK, BGWHITE);
  int i = SCREEN_W*2, j, k;
  for(j = 2; j < SCREEN_H; j++)
  {
    i++;
    for(k = 1; k < SCREEN_W-1; k++)
    if(grid_buf[i++] == 1)
    {
      locate(j, k+1);
      putchar(32);
    }
    i++;
  }
  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  
  locate(food.x, food.y);
  printf("%c", food.type);
  
  //clear old snake
  for(i = 0; i < snake.length-1; i++)
  {
    locate(snake.body_x[i], snake.body_y[i]);
    putchar(' ');
  }
  
  int x = snake.x;
  int y = snake.y;
  //int i;
  fprintf(stdout, "\e%@\e(0");	//enter the Alternative Char Set (ACS) Mode 
  
  for(i = snake.length-1; i >=0; i--)
  {
    locate(x, y);
    snake.body_x[i] = x;
    snake.body_y[i] = y;
    switch(snake.body[i])
    {
      case UP:
	printf("%c", SNAKE_UP_DOWN);
	break;
      case DOWN:
	printf("%c", SNAKE_UP_DOWN);
	break;
      case LEFT_1:
	printf("%c", SNAKE_LEFT_RIGHT_1);
	break;
      case LEFT_2:
	printf("%c", SNAKE_LEFT_RIGHT_2);
	break;
      case RIGHT_1:
	printf("%c", SNAKE_LEFT_RIGHT_1);
	break;
      case RIGHT_2:
	printf("%c", SNAKE_LEFT_RIGHT_2);
	break;
    }//end switch
    if(i == 0) break;
    else
    {
      if(snake.body[i] != snake.body[i-1])
      {
	if(snake.body[i-1] == UP || snake.body[i-1] == DOWN)
	{
	  if(snake.body[i] == LEFT_1 || snake.body[i] == LEFT_2) y++;
	  else y--;
	}
	else if(snake.body[i-1] == LEFT_1 || snake.body[i-1] == LEFT_2 ||
	        snake.body[i-1] == RIGHT_1|| snake.body[i-1] == RIGHT_2)
	{
	  if(snake.body[i] == UP) x++;
	  else x--;
	}
      }
      else
      {
	if(snake.body[i-1] == UP) x++;
	else if(snake.body[i-1] == DOWN) x--;
	else if(snake.body[i-1] == LEFT_1 || snake.body[i-1] == LEFT_2) y++;
	else if(snake.body[i-1] == RIGHT_1 || snake.body[i-1] == RIGHT_2) y--;
      }
    }
  }
  fprintf(stdout, "\e(B");		//exit Alternative Char Set (ACS) Mode
  fflush(stdout);
  //check for body collision
  for(i = 0; i < snake.length-1; i++)
  {
    if(snake.body_x[i] == snake.body_x[snake.length-1] &&
       snake.body_y[i] == snake.body_y[snake.length-1])
    { game_over(); break; }
  }
}

/*
 * Create new random food for the snake
 */
void make_food()
{
make:
  food.x = 2 + rand() % (SCREEN_H-3);
  food.y = 2 + rand() % (SCREEN_W-3);
  food.type = 1 + rand() % 30;
  /* make sure there is no wall here */
  if(grid_buf[(SCREEN_W*food.x)+food.y-1])
    goto make;
  
  if(food.type == 1) 	  { food.type = 35; food.score_inc = 3; }	//(#)
  else if(food.type == 2) { food.type = 36; food.score_inc = 4; }	//($)
  else if(food.type == 3) { food.type = 37; food.score_inc = 2; }	//(%)
  else if(food.type == 4) { food.type = 42; food.score_inc = 1; }	//(*)
  else if(food.type == 5) { food.type = 63; food.score_inc = 3; }	//(?)
  else if(food.type == 6) { food.type = 64; food.score_inc = 5; }	//(@)
  else if(food.type == 7) { food.type = 126; food.score_inc = 5;}	//(~)
  else if(food.type == 8) { food.type = 59; food.score_inc = 4; }	//(;)
  else if(food.type == 9) { food.type = 38; food.score_inc = 5; }	//(&)
  else { food.type = 42; food.score_inc = 1; }	//(*)
}

/*
 * Exit the game safely, clearing the screen and restoring the terminal.
 */
void exit_game()
{
  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  clearScreen();
  restoreTerminal();
  setScreenColors(WHITE, BG_COLOR[COLOR_WINDOW]);
  fprintf(stdout, "\x1b[24m");
  fflush(stdout);
  //turn the cursor on
  printf("\e[?25h");
  exit(0);
}

/*
 * Show the "GAME OVER" box
 */
void game_over()
{
  char *file_name;
  FILE *file;
  int y = (SCREEN_W/2)-14;
  int x = (SCREEN_H/2)-3;
  int w = y+28, h = x+5;
  int i;
  if(!score) goto show_game_over_box;
  
  if(!(pass = getpwuid(geteuid())))
  {
    goto show_game_over_box;
  }
  file_name = (char *) malloc(strlen(pass->pw_dir)+10);
  if(!file_name)
  {
    msgBox("Error: Insufficient memory", OK, ERROR);
    goto show_game_over_box;
  }
  strcpy(file_name, pass->pw_dir);
  strcat(file_name, "/");
  strcat(file_name, ".gosnake");
  if(!(file = fopen(file_name, "r+")))
  {
    //create a new highscores file
    if(!(file = fopen(file_name, "w+")))
    {
      goto show_game_over_box;
    }
    for(i = 0; i < 10; i++)
      fprintf(file, "%4d NONE             0000\n", i+1);
    fflush(file); rewind(file);
  }
  
  //read highscores
  char buf[40];
  char score_str[5];
  int index;
  for(i = 0; i < 10; i++)
  {
    if(!fgets(buf, 40, file)) break;
    buf[26] = '\0';
    score_str[0] = buf[22]; score_str[1] = buf[23]; score_str[2] = buf[24];
    score_str[3] = buf[25]; score_str[4] = '\0';
    if(score >= atoi(score_str))
    {
      //turn the cursor on
      printf("\e[?25h");
      index = i;
      inputBox("Enter your name:      ", "New Highscore!");
      if(!strlen(input)) strcpy(input, "NONE\0");
      if(strlen(input) > 16) input[16] = '\0';
      //turn the cursor off
      printf("\e[?25l");
      fseek(file, 0, SEEK_END);
      long file_pos = ftell(file);
      char *buf2 = (char *)malloc(file_pos);
      char *buf3 = (char *)malloc(40);
      if(!buf2 || !buf3)
      {
	msgBox("Error: Insufficient memory", OK, ERROR);
	goto show_game_over_box;
      }
      rewind(file);
      if(fread(buf2, 1, file_pos, file) != file_pos)
      {
	msgBox("Error reading highscores file", OK, ERROR);
	goto show_game_over_box;
      }
      rewind(file);
      i = 0;
      if(index)
      {
	buf3 = strtok(buf2, "\n");
	buf3[3] = '1';
	fprintf(file, "%s\n", buf3);
	for(i = 1; i < index; i++)
	{
	  buf3 = strtok(NULL, "\n");
	  buf3[3] = i+'1';
	  fprintf(file, "%s\n", buf3);
	}
      }
      
      fprintf(file, "%4d %*s %04d\n", index+1, -16, input, score);
      fflush(file);
      if(index) index++;
      else 
      {
	buf3 = strtok(buf2, "\n");
	buf3[3] = '2';
	fprintf(file, "%s\n", buf3);
	index = 2;
      }
      
      for(i = index; i < 10; i++)
      {
	buf3 = strtok(NULL, "\n");
	if(i == 9) { buf3[2] = '1'; buf3[3] = '0'; }
	else buf3[3] = i+'1';
	fprintf(file, "%s\n", buf3);
      }
      
      fflush(file); rewind(file);
      //draw box
      y = (SCREEN_W/2)-15;
      x = (SCREEN_H/2)-7;
      w = y+30, h = x+13;
      drawBox(x, y, h, w, "HIGHSCORES", YES);
      //read highscores
      for(i = 0; i < 10; i++)
      {
	if(!fgets(buf, 40, file)) break;
	buf[26] = '\0';
	locate(x+i+2, y+2);
	printf("%s", buf);
      }
      fflush(stdout);
      //loop for user pressing ENTER, SPACE, or ESC
      while(1)
      {
	char c = getKey();
	if(c == ENTER_KEY || c == SPACE_KEY || c == ESC_KEY) break;
      }
      fclose(file);
      break;
    }
  }
  show_menu("GoSnake");
  return;
show_game_over_box:
  show_menu("GAME OVER");
  return;
}

void load_level(int level)
{
  LEVEL = level;
  MAX_SCORE = DEFAULT_MAX_SCORE;
  speed = BASE_SPEED;
  score = level_score = 0;
  snake.length = 3;
  snake.x = SCREEN_H-5;
  snake.y = (SCREEN_W/2);
  int i;
  for(i = 0; i < snake.length; i++) snake.body[i] = UP;
  snake.body_x[2] = snake.x;   snake.body_y[2] = snake.y;
  snake.body_x[1] = snake.x+1; snake.body_y[1] = snake.y;
  snake.body_x[0] = snake.x+2; snake.body_y[0] = snake.y;
  make_grid(LEVEL);	//make grid for new level
  make_food();
  drawBox(1, 1, SCREEN_H, SCREEN_W, NULL, YES);
  return;
}